// package softcomputing;
import java.awt.*;
import java.util.*;
import java.lang.*;
import java.io.*;
import java.awt.image.*;

public class GraphView extends OOCanvas implements Observer {

    private static final int GRIDCOLOR = 0xff999999;
    private static final int BACKCOLOR = 0xff000000;
    private static final int PADCOLOR  = 0xff00ff00;

    // the graph associated with this view
    private Graph graph_;
    public Graph graph() { return graph_; }
    public void graph(Graph aGraph) { graph_ = aGraph; }

    // the width of the graph in pixels
    private int graphWidth_;
    public int graphWidth() { return graphWidth_; }
    public void graphWidth(int aWidth) { graphWidth_ = aWidth; }

    // the height of the graph in pixels
    private int graphHeight_;
    public int graphHeight() { return graphHeight_; }
    public void graphHeight(int aHeight) { graphHeight_ = aHeight; }

    // the size of each element in pixels
    private Point elementSize_;
    public Point elementSize() { return elementSize_ ; }
    public void elementSize(Point theSize) { elementSize_ = theSize ; }

    // the insets in pixels of the actual graph in the view
    private Insets graphInsets_;
    public Insets graphInsets() { return graphInsets_ ; }
    public void graphInsets(Insets theInsets) { graphInsets_ = theInsets ; }

    private GraphView(OOView theView) {
        this.theView(theView);
        this.initialize();
    }

    public GraphView(int width, int height, Graph aGraph, OOView theView) {
        this(theView);
        this.graphWidth(width);
        this.graphHeight(height);
        this.graph(aGraph);
        this.hookupObservers();
    }

    private void initialize() {
        System.out.println("init GraphView");
    }

    public void initializeGraphics() {
        this.determineScaling();
    }

    public void hookupObservers() {
        this.graph().addObserver(this);
    }

 // added
    public double[] ySeries() {
        return this.graph().ySeries();
    }

    public void determineScaling() {

        int newWidth =  (int)  (0.7 * this.graphWidth());
        int newHeight = (int)  (0.7 * this.graphHeight());

        // center it

        int xDelta = (this.graphWidth()  - newWidth)  / 2;
        int yDelta = (this.graphHeight() - newHeight) / 2;

        this.graphInsets(new Insets(yDelta, xDelta,
                          yDelta+newHeight, xDelta+newWidth));
        System.out.println("Insets: top = " + this.graphInsets().top +
                               " bottom = " + this.graphInsets().bottom +
                                 " left = " + this.graphInsets().left +
                                " right = " + this.graphInsets().right);
    }

    public void drawTitle(Graphics g) {

        if(this.graphInsets()==null) return;

        g.setColor(Color.black);

        Font theFont = new Font("TimesRoman", Font.PLAIN, 10);
        FontMetrics theMetrics = Toolkit.getDefaultToolkit().getFontMetrics(theFont);
        int titleWidth = theMetrics.stringWidth(this.graph().title());

        int xPos = this.graphInsets().left + (this.graphInsets().right -
                                 this.graphInsets().left - titleWidth) / 2;
        int yPos = this.graphInsets().top - 5;

        g.drawString(this.graph().title(), xPos, yPos);
    }



    public void drawFrame(Graphics g) {

        g.setColor(new Color(GraphView.GRIDCOLOR));
        if (this.graphInsets() == null) {return; }
        g.drawLine(this.graphInsets().left, this.graphInsets().top,
                      this.graphInsets().left, this.graphInsets().bottom);
        g.drawLine(this.graphInsets().left, this.graphInsets().bottom,
                      this.graphInsets().right, this.graphInsets().bottom);
    }

    public void drawGraph(Graphics g) {
        if (this.graphInsets() == null) {return; }

        g.setColor(Color.red);
        int xDelta = this.graphInsets().right  - this.graphInsets().left;
        int yDelta = this.graphInsets().bottom - this.graphInsets().top;

        Point base = new Point(this.graphInsets().left,
                                           this.graphInsets().bottom);
        double xMin = this.graph().minX();
        double yMin = this.graph().minY();
        double xMax = this.graph().maxX();
        double yMax = this.graph().maxY();

        double xRange = (double) (xMax - xMin);
        double yRange = (double) (yMax - yMin);

        double xScale = xDelta / xRange;
        double yScale = yDelta / yRange;

        // now let's draw it

        double[] xData = this.graph().xSeries();
        double[] yData = this.graph().ySeries();

        // get first point
        double xP = (double) xData[0] - xMin;
        double yP = (double) yData[0] - yMin;
        int xPixels = (int) (xP * xScale);
        int yPixels = (int) (yP * yScale);
        Point last = new Point(base.x+xPixels, base.y-yPixels);

        // now the rest
        for (int i=1; i<yData.length; i++) {
            xP = (double) xData[i] - xMin;
            yP = (double) yData[i] - yMin;
            xPixels = (int) (xP * xScale);
            yPixels = (int) (yP * yScale);
            g.drawLine (last.x, last.y, (base.x + xPixels), (base.y - yPixels));
            last.x = base.x + xPixels;
            last.y = base.y - yPixels;
        }

    }


    public void update(Observable obs, Object o) {
        this.repaint();
    }

    public final void drawState(int anX, int aY) {
    }

    public void update(Graphics g) { this.paint(g); }

    public void paint (Graphics g) {
        this.drawFrame(g);
        this.drawTitle(g);
        this.drawGraph(g);
    }

}
