import java.awt.*;
import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;
import java.rmi.*;
import java.rmi.server.*;

public class ServoController extends UnicastRemoteObject implements Runnable, RemoteServoController {

    private PortHandler portHandler;
    public PortHandler portHandler() { return portHandler; }
    public void portHandler (PortHandler theHandler) {portHandler = theHandler; }

    private Thread thread_;
    public Thread thread() { return thread_; }
    public void thread(Thread aThread) {thread_ = aThread; }

    private int servoAPosition_;
    public synchronized int servoAPosition() { return servoAPosition_; }
    public synchronized void servoAPosition(int aPos) {servoAPosition_ = aPos; }

    private int servoBPosition_;
    public synchronized int servoBPosition() { return servoBPosition_; }
    public synchronized void servoBPosition(int aPos) {servoBPosition_ = aPos; }

    private boolean busy_;
    public synchronized boolean busy() { return busy_; }
    public synchronized void busy(boolean aBoolean) {busy_ = aBoolean; }

	/**
	 *  An observable. Necessary since ServoController must
	 *  extend UnicastRemoteObject
	 */
	private ObservableAssistant observableAssistant_;
	public ObservableAssistant observableAssistant() { return observableAssistant_; }
	public void observableAssistant( ObservableAssistant anObservableAssistant ) { observableAssistant_ = anObservableAssistant; }

    public static final int MINX = 50;
    public static final int MAXX = 150;
    public static final int MINY = 80;
    public static final int MAXY = 220;

    ServoController() throws RemoteException {
        this.initialize();
    }

    private void initialize() {
        this.observableAssistant(new ObservableAssistant());
        this.portHandler(new PortHandler());
        this.servoAPosition(127);
        this.servoBPosition(127);
        this.busy(false);
        this.thread(new Thread(this));
        this.thread().start();
    }

    public synchronized void newPositions(int posA, int posB) {
        this.servoAPosition(posA);
        this.servoBPosition(posB);
        this.publicize();
    }

    public void publicize() {
        this.observableAssistant().publicize();
    }

    public void run() {

        SerialPort sPort = this.portHandler().serialPort();
        PortHandler pHdlr = this.portHandler();

        while (true) {


            try {

                sPort.putByte((byte)0xff);
                sPort.putByte((byte)0x00);
                sPort.putByte((byte) this.servoAPosition());

                sPort.putByte((byte)0xff);
                sPort.putByte((byte)0x01);
                // flip the y so down is down by 300-y instead
                // of just y
                sPort.putByte((byte)(300 - this.servoBPosition()));

                try{Thread.sleep(5);} catch(InterruptedException e) {}
                sPort.rxFlush();

            }

            catch (IOException e) {}
        }
    }

    public void updatePositionTo (int posA, int posB) throws RemoteException {
        this.servoAPosition(posA);
        this.servoBPosition(posB);
        this.publicize();
    }

}
