import java.util.*;
import java.lang.*;

class Zoo extends Observable implements Runnable {

    private static int FREIGHT_CHECK_INTERVAL = 12000;

	/**
	 *  The name of this Zoo
	 */
	private String name_;
	public String name() { return name_; }
	public void name( String aString ) { name_ = aString; }

	/**
	 *  The animal container
	 */
	private Hashtable animals_;
	public Hashtable animals() { return animals_; }
	public void animals( Hashtable anHashtable ) { animals_ = anHashtable; }

	/**
	 *  Indicates that the thread associated with this Zoo is busy
	 */
	private boolean busy_;
	public boolean busy() { return busy_; }
	public void busy( boolean aBoolean ) { busy_ = aBoolean; }

	/**
	 *  Setting this to true forces the tread to stop when it completes
	 *  the current action
	 */
	private boolean done_;
	public boolean done() { return done_; }
	public void done( boolean aBoolean ) { done_ = aBoolean; }

	/**
	 *  The thread associated with this Zoo
	 */
	private Thread thread_;
	public Thread thread() { return thread_; }
	public void thread( Thread aThread ) { thread_ = aThread; }

	/**
	 *  starts at 100 and is incremented for each new recieved
	 *  animal and concatenated to the name
	 *   to insure that animal names are unique.
	 */
	private int uniqueNameCounter_;
	private int uniqueNameCounter() { return uniqueNameCounter_; }
	private void uniqueNameCounter( int anInt ) { uniqueNameCounter_ = anInt; }


    Zoo() {
        this.initialize();
    }

    private void initialize() {

        this.busy(false);
        this.done(false);
        this.thread(new Thread(this)); // define the thread but dont start it yet

        this.uniqueNameCounter(100);

        this.animals(new Hashtable());
        this.name("Fred's Zoo");

        Hashtable theAs  = this.animals();
        theAs.put("George", (new Animal("George", 2)));
        theAs.put("Centi", (new Animal("Centi",  43)));
        theAs.put("Wormy", (new Animal("Wormy",  0)));
        theAs.put("Yogi", (new Animal("Yogi",   4)));

    }

    public void startTheEngine(){
        this.publicize();
        this.thread().start();
    }

    public void publicize() {
        this.setChanged();
        this.notifyObservers();
    }

    public int computeFootCount() {
        int totalFootCount = 0;
        Enumeration theAnimals = this.animals().elements();
        while (theAnimals.hasMoreElements()) {
            Animal anAnimal = (Animal) theAnimals.nextElement();
            totalFootCount = totalFootCount +
                                anAnimal.footCount();
        }
        return totalFootCount;
    }


    // added functionality ........

    public Animal checkFreight() {
        String someNewNames[] = {"Bear","Snake","Gorilla","Dolphin","Parot","Kiwi","Lion"};
        int corrospondingFootCounts[] = {4,0,2,0,2,4,4};

        int nameChoice = (int) (Math.random() * 7.0);

        Animal newAnimal = new Animal(someNewNames[nameChoice] + " " + this.uniqueNameCounter(),corrospondingFootCounts[nameChoice]);

        this.uniqueNameCounter(  this.uniqueNameCounter() + 1 );

        return newAnimal;
    }

    public Vector animalNames() {
        Vector theNames = new Vector();
        Enumeration theKeys = this.animals().keys();
        while (theKeys.hasMoreElements()) {
            String aName = (String) theKeys.nextElement();
            theNames.addElement(aName);
        }
        return theNames;
    }

    public int currentPopulation() {
        return this.animals().size();
    }

    public Animal animalForName(String aName) {
        return (Animal) this.animals().get(aName);
    }

    public void run() {

        while (!this.done()) {
            try {
                Thread.sleep(Zoo.FREIGHT_CHECK_INTERVAL);
            }
            catch (InterruptedException e) {
                System.out.println(".. InterruptedException");
            }

            Animal anAnimal = this.checkFreight();
            this.animals().put(anAnimal.name(), anAnimal);
            this.publicize();
        }
    }



}


