import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.awt.image.*;

// v5.0 changes to directly draw without paint, update, etc.
// v6.0 eliminate Points and update states[][] directly from BoolNet.run()
// v7.0 shorten drawElement

class GridTest extends Observable implements Runnable {

    long oldTime = 0;

    private Grid grid_;
    public Grid grid(){ return grid_; }
    public void grid(Grid aGrid) { grid_ = aGrid; }

    private boolean busy_;
    public boolean busy() { return busy_ ; }
    public void busy(boolean aBoolean) { busy_ = aBoolean ; }

    private boolean done_;
    public boolean done() { return done_ ; }
    public void done(boolean aBoolean) { done_ = aBoolean ; }

    private Thread thread_;
    public Thread thread() { return thread_ ; }
    public void thread(Thread aThread) { thread_ = aThread ; }

    private ResultLog resultLog_;
    public ResultLog resultLog() { return resultLog_; }
    public void resultLog(ResultLog theRL) { resultLog_ = theRL; }


    public GridTest() {
        this.initialize();
    }

    private void initialize() {
        this.busy(false);
        this.done(false);
        this.resultLog(new ResultLog());

    }

    public void startTheEngine() {
        if (this.busy()) return;
        this.busy(true);
        this.thread(new Thread(this));
        this.thread().start();
    }


    public void publicize() {
        this.setChanged();
        this.notifyObservers();
    }




    public void run() {

        System.out.println("Begin Network Creation");

        this.grid().createBooleanNetwork();

        System.out.println("Creation Complete");

        while (!(this.done())) {

            oldTime = System.currentTimeMillis(); // for debugging durations

            // loop thru to make the model compute the intermediate state
            Enumeration theElts = this.grid().booleanElements().elements();

            while (theElts.hasMoreElements()) {
                BooleanElement theCurrentElement = (BooleanElement) theElts.nextElement();
                int theNextState = theCurrentElement.computeIntermediateState();
            }

            // now copy the intermediate state to the output

            theElts = this.grid().booleanElements().elements();

            while (theElts.hasMoreElements()) {
                BooleanElement theCurrentElement = (BooleanElement) theElts.nextElement();
                theCurrentElement.outputState(theCurrentElement.intermediateState());
            }

            // now create the result BitSet and add it to the ResultLog

            BitSet theResults = new BitSet(this.grid().booleanElements().size());
            int bitPosition = 0;

            theElts = this.grid().booleanElements().elements();

            while (theElts.hasMoreElements()) {
                BooleanElement theCurrentElement = (BooleanElement) theElts.nextElement();
                int interimState = theCurrentElement.intermediateState();
                if (interimState == 0) theResults.set(bitPosition);
                else                   theResults.clear(bitPosition);
                bitPosition++;
            }

            this.resultLog().addResult(theResults);

            this.resultLog().latestHasDuplicatesBackwards();


         //   this.displayDuration("Model Manipulation");

            this.grid().cycleComplete();

            try{Thread.sleep(20);} catch(InterruptedException e) {}

       //     this.displayDuration("Painting");

        }

        this.busy(false);
    }

    public void displayDuration(String phase) {
        long now = System.currentTimeMillis();
        System.out.println("Duration of " + phase + " Phase =  " + (now - oldTime));
        oldTime = now;
    }

}
