// package softcomputing;
import java.awt.*;
import java.util.*;
import java.lang.*;
import java.io.*;
import java.awt.image.*;
import java.awt.event.*;

public class GridView extends Canvas implements Observer, MouseListener {

    private static final int GRIDCOLOR = 0xff444444;
    private static final int BACKCOLOR = 0xff000000;
    private static final int PADCOLOR  = 0xff00ff00;

    // the grid associated with this view
    private Grid grid_;
    public Grid grid() { return grid_; }
    public void grid(Grid aGrid) { grid_ = aGrid; }

    // the actual pixels of the view
    private int[] pixels_;
    public int[] pixels() { return pixels_; }
    public void pixels(int[] thePixels) { pixels_ = thePixels; }

    // the width of the grid in pixels
    private int gridWidth_;
    public int gridWidth() { return gridWidth_; }
    public void gridWidth(int aWidth) { gridWidth_ = aWidth; }

    // the height of the grid in pixels
    private int gridHeight_;
    public int gridHeight() { return gridHeight_; }
    public void gridHeight(int aHeight) { gridHeight_ = aHeight; }

    // the image of the grid
    private Image image_;
    public Image image() { return image_; }
    public void image(Image anImage) {image_ = anImage; }

    // the size of each element in pixels
    private Point elementSize_;
    public Point elementSize() { return elementSize_ ; }
    public void elementSize(Point theSize) { elementSize_ = theSize ; }

    // the offset in pixels of the actual grid in the view
    private Point gridOffset_;
    public Point gridOffset() { return gridOffset_ ; }
    public void gridOffset(Point theOffset) { gridOffset_ = theOffset ; }

    private GridView() {
        this.initialize();
    }

    public GridView(Grid aGrid) {
        this();
        this.grid(aGrid);
        this.hookupObservables();
    }

    private void initialize() {
        this.addMouseListener(this);
        System.out.println("init GridView");
    }

    public void initializeGraphics() {

        this.determineScaling(); // determine elementSize

        int theWidth  = (this.grid().gridSize().x * this.elementSize().x) + 1;
        int theHeight = (this.grid().gridSize().y * this.elementSize().y) + 1;
        this.gridWidth(theWidth);
        this.gridHeight(theHeight);

        this.pixels(new int[theWidth * theHeight]);

        for (int i=0; i<theWidth*theHeight; i++) {
            this.pixels()[i] = GridView.BACKCOLOR;
        }

        this.image(this.createImage(
                   new MemoryImageSource(
                        theWidth, theHeight, this.pixels(), 0, theWidth)));
        this.drawGrid(); // do this only once
        this.image().flush();

    }

    public void hookupObservables() {
        this.grid().addObserver(this);
    }

    public void determineScaling() {

        int imax = this.grid().gridSize().y;
        int jmax = this.grid().gridSize().x;

        int bX = (int) (((this.size().width)  - 1) / new Integer(jmax).floatValue());
        int bY = (int) (((this.size().height) - 1) / new Integer(imax).floatValue());
        this.elementSize(new Point( bX, bY));
        int xOffset = (int) ((this.size().width
                                       - ((jmax * bX) + 1)) / 2.0 );
        int yOffset = (int) ((this.size().height
                                       - ((imax * bY) + 1)) / 2.0 );
        this.gridOffset(new Point(xOffset, yOffset));
    }

    public void drawGrid() {

        for (int i=0; i<this.gridHeight(); i+=this.elementSize().y) {
            for (int j=0; j<this.gridWidth(); j++) {
                this.pixels()[i * this.gridWidth() + j] = GridView.GRIDCOLOR;
            }
        }

        for (int i=0; i<this.gridHeight(); i++) {
            for (int j=0; j<this.gridWidth(); j+=this.elementSize().x) {
                this.pixels()[i * this.gridWidth() + j] = GridView.GRIDCOLOR;
            }
        }
    }

    public void drawStates() {
        Enumeration theElts = this.grid().booleanElements().elements();
        while (theElts.hasMoreElements()) {
            BooleanElement theCurrentElement = (BooleanElement) theElts.nextElement();
            int theState = theCurrentElement.outputState();
            this.drawElement(theCurrentElement.gridLocation().x,
                                 theCurrentElement.gridLocation().y, theState);
        }
    }


    public final void drawState(int anX, int aY) {

 //       this.drawElement(anX, aY, this.grid().states()[anX][aY]);
    }

    public void drawElement(int anX, int aY, int state) {

        // compute some variables

        int padWidth = this.elementSize().x-1;
        int padHeight = this.elementSize().y-1;

        int xOffset = (anX * elementSize().x)+1;
        int yOffset = (aY  * elementSize().y)+1;

        int thePixels[] = this.pixels();

        if (state==1) {

            int yPos = yOffset * this.gridWidth();
            for (int i=0; i<padHeight; i++) {
                for (int j=0; j<padWidth; j++) {
                    thePixels[yPos + j + xOffset] = GridView.PADCOLOR;
                }
                yPos += this.gridWidth();
            }
        }

        else {

            int yPos = yOffset * this.gridWidth();
            for (int i=0; i<padHeight; i++) {
                for (int j=0; j<padWidth; j++) {
                    thePixels[yPos + j + xOffset] = GridView.BACKCOLOR;
                }
                yPos += this.gridWidth();
            }
        }
    }

    public void update(Graphics g) {}  // disable normal paint cycle
    public void paint (Graphics g) {}

    public void update(Observable obs, Object o) {
        this.showIt();
    }

    public void showIt() {
        this.drawStates();
        Graphics g = this.getGraphics();
        g.drawImage(this.image(),0,0,this);
        g.dispose();
        this.image().flush();
    }

    public void mousePressed(MouseEvent e) {
        e.translatePoint(- this.gridOffset().x, - this.gridOffset().y);
        int xLoc = e.getX() / this.elementSize().x;
        int yLoc = e.getY() / this.elementSize().y;
        System.out.println("x = " + xLoc + "  y = " + yLoc);
        BooleanElement theBE = this.grid().findElementAt(new Point(xLoc, yLoc));
        System.out.println("State    = " + theBE.outputState());
        System.out.println("BitSet   = " + theBE.booleanRule().toString());
        System.out.println("Self     = " + theBE);
        System.out.println("Siblings = " + theBE.siblingA() + " " + theBE.siblingB());
        System.out.println(" ");

    }

    public void mouseClicked(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}

}
