import java.util.*;

class Thermostat implements Runnable {

	/**
	 *  Senses the ambient temperature
	 */
	private TemperatureSensor temperatureSensor_;
	public TemperatureSensor temperatureSensor() { return temperatureSensor_; }
	public void temperatureSensor( TemperatureSensor aTemperatureSensor ) { temperatureSensor_ = aTemperatureSensor; }

	/**
	 *  The Air Conditioner controlled by the Thermostat
	 */
	private AirConditioner airConditioner_;
	public AirConditioner airConditioner() { return airConditioner_; }
	public void airConditioner( AirConditioner anAirConditioner ) { airConditioner_ = anAirConditioner; }

	/**
	 *  The heater controlled by the Thermostat
	 */
	private Heater heater_;
	public Heater heater() { return heater_; }
	public void heater( Heater anHeater ) { heater_ = anHeater; }

	/**
	 *  Indicates that the thread associated with this Thermostat is busy
	 */
	private boolean busy_;
	public boolean busy() { return busy_; }
	public void busy( boolean aBoolean ) { busy_ = aBoolean; }

	/**
	 *  Setting this to true forces the tread to stop when it completes
	 *  the current action
	 */
	private boolean done_;
	public boolean done() { return done_; }
	public void done( boolean aBoolean ) { done_ = aBoolean; }

	/**
	 *  The thread associated with this Thermostat
	 */
	private Thread thread_;
	public Thread thread() { return thread_; }
	public void thread( Thread aThread ) { thread_ = aThread; }



    public Thermostat() {
        this.initialize();
    }

    private void initialize() {

        this.temperatureSensor(new TemperatureSensor());
        this.airConditioner(new AirConditioner());
        this.heater(new Heater());

        this.busy(false);
        this.done(false);
        this.thread(new Thread(this)); // dont start it yet

    }

    public void startTheEngine(){
        this.thread().start();
    }


    public void temperatureHasChanged(int newTemperature) {
        this.temperatureSensor().currentTemperature(newTemperature);
    }

    public void runTemperatureAlgorithm() {

        int newTemperature = this.temperatureSensor().currentTemperature();

        if (newTemperature < this.heatBelowTemperature()) {
            this.heater().turnOn();
        }
        else {
            this.heater().turnOff();
        }

        if (newTemperature > this.coolAboveTemperature()) {
            this.airConditioner().turnOn();
        }
        else {
            this.airConditioner().turnOff();
        }
    }

    public int heatBelowTemperature() {
        return (new Integer(Configuration.instance()
                            .valueOf("HEATBELOW"))).intValue();

    }

    public int coolAboveTemperature() {
        return (new Integer(Configuration.instance()
                            .valueOf("COOLABOVE"))).intValue();
    }

    public void run() {

        this.busy(true);

        while (!this.done()) {
            this.runTemperatureAlgorithm();
            try {
                Thread.sleep(500);
            }
            catch (InterruptedException e) {
                System.out.println("uh .. oh");
            }
        }

        this.busy(false);
    }

}


