
import java.util.*;

public class TravelPlanGenerator extends Observable {

    private  FlyByNiteReservationSystem reservationSystem_;
    public FlyByNiteReservationSystem reservationSystem() { return reservationSystem_; }
    public void reservationSystem(FlyByNiteReservationSystem aFlyByNiteReservationSystem) { reservationSystem_ = aFlyByNiteReservationSystem; }

	private Vector plans_;
	public Vector plans() { return plans_; }
	public void plans( Vector aVector ) { plans_ = aVector; }

    public TravelPlanGenerator() {
        this.initialize();
    }

    private TravelPlanGenerator(FlyByNiteReservationSystem theReservationSystem) {
        this();
        this.reservationSystem(theReservationSystem);
    }

    private void initialize() {
        // reservation system loaded by TravelPlanGeneratorView
        this.plans(new Vector());
    }

    public void generateTravelPlansFrom(TravelPlanRequest theTPR) {

        this.plans(new Vector());

        // first try the direct flights
        this.generateDirectTravelPlansFrom(theTPR);

        // now let the world know
        this.publicize();
    }

    public void generateDirectTravelPlansFrom(TravelPlanRequest theTPR) {

        // first get the Airport
        Airport requestedAirport = this.reservationSystem()
                            .findAirportWithName(theTPR.destinationPoint());

        // get legs with valid departure points
        Vector theDepartures   = this
                      .flightLegsWithDeparturePoint(theTPR.departurePoint());

        // loop thru the departures looking for the correct destination
        Enumeration theOC = theDepartures.elements();
        while(theOC.hasMoreElements()) {
            FlightLeg aFL = (FlightLeg) theOC.nextElement();
            if(aFL.destinationPoint().equals(requestedAirport)) {
                TravelPlan aTP = new TravelPlan();
                aTP.addFlightLeg(aFL);
                this.plans().addElement(aTP);
            }
        }
    }

    public Vector flightLegsWithDeparturePoint(String theDeparturePoint) {

        // first get the Airport
        Airport requestedAirport = this.reservationSystem()
                                  .findAirportWithName(theDeparturePoint);

        // now make a vector of valid FlightLegs
        Vector theFLs = new Vector();
        Enumeration theOC = this.reservationSystem().flightLegs().elements();
        while(theOC.hasMoreElements()) {
            FlightLeg aFlightLeg = (FlightLeg) theOC.nextElement();
            if (aFlightLeg.departurePoint().equals(requestedAirport)) {
                theFLs.addElement(aFlightLeg);
            }
        }
        return theFLs;
    }

    public Vector flightLegsWithDestinationPoint(String theDestinationPoint) {
        // first get the Airport
        Airport requestedAirport = this.reservationSystem()
                                  .findAirportWithName(theDestinationPoint);

        // now make a vector of valid FlightLegs
        Vector theFLs = new Vector();
        Enumeration theOC = this.reservationSystem().flightLegs().elements();
        while(theOC.hasMoreElements()) {
            FlightLeg aFlightLeg = (FlightLeg) theOC.nextElement();
            if (aFlightLeg.destinationPoint().equals(requestedAirport)) {
                theFLs.addElement(aFlightLeg);
            }
        }
        return theFLs;
    }

    public void generateOld() {
        this.plans(new Vector());

        TravelPlan aTP = new TravelPlan();
        aTP.addFlightLeg((FlightLeg) this.reservationSystem().flightLegs().elementAt(1));
        aTP.addFlightLeg((FlightLeg) this.reservationSystem().flightLegs().elementAt(2));
        this.plans().addElement(aTP);
        this.publicize();
    }

    public void flightLegSelected(String theLeg) {
        System.out.println("leg " + theLeg);
    }


    public void publicize() {
        this.setChanged();
        this.notifyObservers();
    }

}