import java.awt.*;
import java.awt.event.*;
import java.util.*;

class ATMView extends SCView {

	/**
	 *  The ATM associated with this View
	 */
	private ATM atm_;
	public ATM atm() { return atm_; }
	public void atm( ATM anATM ) { atm_ = anATM; }

	/**
	 *  The dollar value of the withdraw or deposit
	 */
	private String amount_;
	private String amount() { return amount_; }
	private void amount( String aString ) { amount_ = aString; }

	/**
	 *  The name of the customer.
	 */
	private String name_;
	private String name() { return name_; }
	private void name( String aString ) { name_ = aString; }

	/**
	 *  The PIN number of the customer.
	 */
	private String pin_;
	private String pin() { return pin_; }
	private void pin( String aString ) { pin_ = aString; }

	/**
	 *  If anything goes wrong with the transaction,
	 *  this flag gets set to true and everything goes
	 *  back to the beginning.
	 */
	private boolean transactionAborted_;
	private boolean transactionAborted() { return transactionAborted_; }
	private void transactionAborted( boolean aBoolean ) { transactionAborted_ = aBoolean; }





    private ATMView(){
        super("ATM View");
        this.initialize();
    }

    public static void main(String argv[]){

        ATMView theATMView = new ATMView();
        theATMView.show();
        theATMView.startATMSequence();
    }

    private void initialize(){

        this.atm(new ATM());
        this.atm().addObserver(this);

        this.setLayout(null);

        this.transactionAborted(false);

        this.initMenu();

        this.initWidgets();

        this.setSize(450,360);
        this.setLocation(80,80);

    }
    private void initWidgets(){

        this.addWidget("response TextArea",new ResponseTextArea());

        this.addWidget("enter PIN Label", new EnterPINLabel());
        this.addWidget("enter PIN TextField", new EnterPINTextField(this));
        this.addWidget("done entering PIN Button", new DoneEnteringPINButton(this));

        this.addWidget("select transaction type Label", new SelectTypeLabel());
        this.addWidget("withdraw type Label", new WithdrawTypeLabel());
        this.addWidget("withdraw type Button", new WithdrawTypeButton(this));
        this.addWidget("deposit type Label", new DepositTypeLabel());
        this.addWidget("deposit type Button", new DepositTypeButton(this));

        this.addWidget("enter name Label", new EnterNameLabel());
        this.addWidget("enter name TextField", new EnterNameTextField(this));
        this.addWidget("done entering name Button", new DoneEnteringNameButton(this));

        this.addWidget("withdraw Label", new WithdrawLabel());
        this.addWidget("withdraw amount Label", new WithdrawAmountLabel());
        this.addWidget("withdraw amount TextField", new WithdrawAmountTextField(this));
        this.addWidget("done entering withdraw amount Button", new DoneEnteringWithdrawAmountButton(this));

        this.addWidget("deposit Label", new DepositLabel());
        this.addWidget("deposit amount Label", new DepositAmountLabel());
        this.addWidget("deposit amount TextField", new DepositAmountTextField(this));
        this.addWidget("done entering deposit amount Button", new DoneEnteringDepositAmountButton(this));

        this.addWidget("do you want another Label", new DoYouWantAnotherLabel());
        this.addWidget("yes Label", new YesLabel());
        this.addWidget("no Label", new NoLabel());
        this.addWidget("yes Button", new YesButton(this));
        this.addWidget("no Button", new NoButton(this));

        this.hideAllWidgets();

    }

    private void clearAllTextComponents(){
        ((TextField) this.getWidget("enter name TextField")).setText("");
        ((TextField) this.getWidget("enter PIN TextField")).setText("");
        ((TextField) this.getWidget("withdraw amount TextField")).setText("");
        ((TextField) this.getWidget("deposit amount TextField")).setText("");

        ((TextArea) this.getWidget("response TextArea")).setText("");
    }
    private void hideAllWidgets(){
        this.hideWidgetsForEnteringName();
        this.hideWidgetsForEnteringPIN();
        this.hideWidgetsForSelectingType();
        this.hideWidgetsForEnteringWithdrawAmount();
        this.hideWidgetsForEnteringDepositAmount();
        this.hideWidgetsForDoYouWantAnother();
    }

    private void startATMSequence(){
        this.transactionAborted(false);
        this.showWidgetsForEnteringName();
    }
    private void sendWithdraw(){

        this.atm().enterWithdrawal(this.name(), this.pin(), (Double.valueOf(this.amount())).doubleValue());


    }



    private void sendDeposit(){

    	this.atm().enterDeposit(this.name(), this.pin(), (Double.valueOf(this.amount())).doubleValue());


    }


    public void update(Observable obs, Object o) {

        String theString = ((String) o) + "\n";
        if(theString.equals("ACCOUNT_INVALID\n")){
            ((TextArea) this.getWidget("response TextArea")).setText("Account Invalid\nTransaction Terminated");
            try{ Thread.sleep(2000); }
            catch(InterruptedException ie){}
            this.transactionAborted(true);
            return;
        }
        ((TextArea) this.getWidget("response TextArea")).append(theString);


        System.out.println(">> " + theString);
    }

   public void actionPerformed(ActionEvent e){
        String actionCommand = e.getActionCommand();
        if( actionCommand.equals("Exit") ){
            this.exit();
            return;
        }
    }

    private void initMenu(){
        MenuBar aMenuBar = new MenuBar();
            Menu aMenu = new Menu("File");
                MenuItem aMenuItem = new MenuItem("Exit");
                aMenuItem.addActionListener(this);
                aMenuItem.setActionCommand("Exit");
            aMenu.add(aMenuItem);
            aMenuBar.add(aMenu);


        this.setMenuBar(aMenuBar);
    }

    // methods called by widgets

    public void doneEnteringName(){

        this.name( ((TextField) this.getWidget("enter name TextField")).getText());

        this.hideWidgetsForEnteringName();
        this.showWidgetsForEnteringPIN();
    }

    public void doneEnteringPIN(){

        this.pin( ((TextField) this.getWidget("enter PIN TextField")).getText());

        this.hideWidgetsForEnteringPIN();
        this.showWidgetsForSelectingType();
    }
    public void withdrawTypeChosen(){

        this.hideWidgetsForSelectingType();
        this.showWidgetsForEnteringWithdrawAmount();
    }
    public void depositTypeChosen(){

        this.hideWidgetsForSelectingType();
        this.showWidgetsForEnteringDepositAmount();
    }
    public void doneEnteringDepositAmount(){

        this.amount( ((TextField) this.getWidget("deposit amount TextField")).getText());
	try{
		this.sendDeposit();
		if(this.transactionAborted()){
		    this.clearAllTextComponents();
		    this.hideAllWidgets();
		    this.startATMSequence();
		    return;
		}
 		this.hideWidgetsForEnteringDepositAmount();
   		this.showWidgetsForDoYouWantAnother();
	}
	catch(Exception e){}

    }
    public void doneEnteringWithdrawAmount(){

        this.amount( ((TextField) this.getWidget("withdraw amount TextField")).getText());
 	try{
		this.sendWithdraw();
		if(this.transactionAborted()){
		    this.clearAllTextComponents();
		    this.hideAllWidgets();
		    this.startATMSequence();
            return;
		}
		this.hideWidgetsForEnteringWithdrawAmount();
		this.showWidgetsForDoYouWantAnother();
	}
	catch(Exception e){}





    }
    public void doneWithNoChosen(){

        this.clearAllTextComponents();

        this.hideWidgetsForDoYouWantAnother();
        this.showWidgetsForEnteringName();
    }
    public void doneWithYesChosen(){

        this.clearAllTextComponents();

        this.hideWidgetsForDoYouWantAnother();
        this.showWidgetsForSelectingType();
    }

    //  methods for showing and hiding widgets between stages
    private void showWidgetsForEnteringName(){
        this.getWidget("enter name Label").show();
        this.getWidget("enter name TextField").show();
        this.getWidget("done entering name Button").show();

        this.getWidget("enter name TextField").requestFocus();
    }

    private void hideWidgetsForEnteringName(){
        this.getWidget("enter name Label").hide();
        this.getWidget("enter name TextField").hide();
        this.getWidget("done entering name Button").hide();
    }

    private void showWidgetsForEnteringPIN(){
        this.getWidget("enter PIN Label").show();
        this.getWidget("enter PIN TextField").show();
        this.getWidget("done entering PIN Button").show();

        this.getWidget("enter PIN TextField").requestFocus();
    }

    private void hideWidgetsForEnteringPIN(){
        this.getWidget("enter PIN Label").hide();
        this.getWidget("enter PIN TextField").hide();
        this.getWidget("done entering PIN Button").hide();
    }

    private void showWidgetsForSelectingType(){
        this.getWidget("select transaction type Label").show();
        this.getWidget("withdraw type Label").show();
        this.getWidget("deposit type Label").show();
        this.getWidget("withdraw type Button").show();
        this.getWidget("deposit type Button").show();

    }

    private void hideWidgetsForSelectingType(){
        this.getWidget("select transaction type Label").hide();
        this.getWidget("withdraw type Label").hide();
        this.getWidget("deposit type Label").hide();
        this.getWidget("withdraw type Button").hide();
        this.getWidget("deposit type Button").hide();
    }

    private void showWidgetsForEnteringWithdrawAmount(){
        this.getWidget("withdraw amount Label").show();
        this.getWidget("withdraw amount TextField").show();
        this.getWidget("withdraw Label").show();
        this.getWidget("done entering withdraw amount Button").show();

        this.getWidget("withdraw amount TextField").requestFocus();
    }

    private void hideWidgetsForEnteringWithdrawAmount(){
        this.getWidget("withdraw amount Label").hide();
        this.getWidget("withdraw amount TextField").hide();
        this.getWidget("withdraw Label").hide();
        this.getWidget("done entering withdraw amount Button").hide();
    }

    private void showWidgetsForEnteringDepositAmount(){
        this.getWidget("deposit amount Label").show();
        this.getWidget("deposit amount TextField").show();
        this.getWidget("deposit Label").show();
        this.getWidget("done entering deposit amount Button").show();

        this.getWidget("deposit amount TextField").requestFocus();
    }

    private void hideWidgetsForEnteringDepositAmount(){
        this.getWidget("deposit amount Label").hide();
        this.getWidget("deposit amount TextField").hide();
        this.getWidget("deposit Label").hide();
        this.getWidget("done entering deposit amount Button").hide();
    }

    private void showWidgetsForDoYouWantAnother(){
        this.getWidget("do you want another Label").show();
        this.getWidget("yes Label").show();
        this.getWidget("no Label").show();
        this.getWidget("yes Button").show();
        this.getWidget("no Button").show();

        this.getWidget("yes Button").requestFocus();

    }

    private void hideWidgetsForDoYouWantAnother(){
        this.getWidget("do you want another Label").hide();
        this.getWidget("yes Label").hide();
        this.getWidget("no Label").hide();
        this.getWidget("yes Button").hide();
        this.getWidget("no Button").hide();
    }





    public void windowClosing(WindowEvent we){
        this.exit();
    }



    private void exit(){
        System.out.println("closing ATMView");
        try{ Thread.sleep(500); }
        catch(InterruptedException ie){}
        this.dispose();
        System.exit(0);
    }

}

