
import java.util.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;

class DirectoryDialog extends Dialog implements ActionListener {

    private String separator;
    private FontMetrics fontMetrics;

	/**
	 */
	private DirectoryDialogList directoryDialogList_;
	private DirectoryDialogList directoryDialogList() { return directoryDialogList_; }
	private void directoryDialogList( DirectoryDialogList aDirectoryDialogList ) { directoryDialogList_ = aDirectoryDialogList; }

	/**
	 */
	private Button upButton_;
	private Button upButton() { return upButton_; }
	private void upButton( Button aButton ) { upButton_ = aButton; }

	/**
	 */
	private Button doneButton_;
	private Button doneButton() { return doneButton_; }
	private void doneButton( Button aButton ) { doneButton_ = aButton; }

	/**
	 *  this label will always display the String that is returned by
	 *  getPath() of AWTFileDialog.
	 */
	private Label choiceLabel_;
	private Label choiceLabel() { return choiceLabel_; }
	private void choiceLabel( Label aLabel ) { choiceLabel_ = aLabel; }


    public DirectoryDialog(Frame f){

        super(f,"Directory Dialog",true);
        this.initialize();
    }

    private void initialize(){
        this.setLayout(null);
        this.setResizable(false);
        this.initWidgets();

        this.fontMetrics = Toolkit.getDefaultToolkit().getFontMetrics(new Font("Courier",0,14));

        this.separator = System.getProperty("file.separator");

        File aFile = new File(".");
        try{
            aFile = new File( aFile.getCanonicalPath() );
        }
        catch(IOException ioe){
            System.out.println("IOException in initialize() of awtfiledialogtestview");
        }

        this.setLabelString(aFile.toString());
        this.listFilesForDirectory( aFile );
        this.setSize(320,410);
        this.setLocation(100,100);
    }

    private void initWidgets(){

        this.directoryDialogList(new DirectoryDialogList(this));
        this.add( this.directoryDialogList() );

        Button aButton = new Button("Done");
        aButton.setSize(90,30);
        aButton.setLocation(10,370);
        aButton.setFont(new Font("Courier",0,14));
        aButton.setActionCommand("Done");
        aButton.addActionListener(this);
        this.doneButton( aButton );
        this.add(this.doneButton());

        aButton = new Button("Up");
        aButton.setSize(90,30);
        aButton.setLocation(110,370);
        aButton.setFont(new Font("Courier",0,14));
        aButton.setActionCommand("Up");
        aButton.addActionListener(this);
        this.upButton( aButton );
        this.add(this.upButton());

        Label aLabel = new Label("");
        aLabel.setSize(300,30);
        aLabel.setLocation(10,30);
        aLabel.setFont(new Font("Courier",0,14));
        aLabel.setAlignment(Label.LEFT);
        this.choiceLabel( aLabel );
        this.add(aLabel);

    }

    private void newList(){
        this.remove(this.directoryDialogList());
        this.directoryDialogList(new DirectoryDialogList(this));
        this.add( this.directoryDialogList() );
    }

   public void actionPerformed(ActionEvent e){
        String actionCommand = e.getActionCommand();
        if( actionCommand.equals("Done") ){
            this.done();
            return;
        }
        if( actionCommand.equals("Up") ){
            this.up();
            return;
        }
        if( actionCommand.equals("test") ){
            this.test();
            return;
        }
    }

    private void test(){

        System.out.println("done in AWTFileDialog");

        File aFile = new File(".");

        String fileNames[] = aFile.list();
        for(int n = 0; n < fileNames.length; n++ ){

            this.directoryDialogList().addItem(fileNames[n]);

            if(fileNames[n].endsWith(".java")){
                System.out.println("next java file name = " + fileNames[n]);
            }
        }

    }

    private void up(){
        this.setWaitingCursor();

        File aFile = new File(this.choiceLabel().getText());

        if( aFile.isDirectory() ){
            aFile = new File(aFile.getParent());
        }
        else{
            aFile = new File(aFile.getParent());
            aFile = new File( aFile.getParent() );
        }
        if(aFile.getParent() == null){
            this.upButton().setEnabled(false);
        }
        else{
            this.upButton().setEnabled(true);
        }

        this.listFilesForDirectory(aFile);
        this.resetCursor();

    }

    private void done(){
       this.hide();
    }

    public void listClicked(String selectedItem ){
        this.setWaitingCursor();
        File aFile = new File(this.choiceLabel().getText());

        if( aFile.list() != null ){
            aFile = new File(aFile.getPath() + separator + selectedItem);
        }
        else{
            aFile = new File(aFile.getParent() + separator + selectedItem);
        }

        if( aFile.getParent() == null){
            this.upButton().setEnabled(false);
        }
        else{
            this.upButton().setEnabled(true);
        }

        try{
            aFile = new File( aFile.getCanonicalPath() );
        }
        catch(IOException ioe){
            System.out.println("IOException in listclicked of awtfiledialog");
        }

        this.setLabelString(aFile.getPath());
        if( aFile.isDirectory() ){
            this.listFilesForDirectory(aFile);
        }
        this.resetCursor();
    }

    private void listFilesForDirectory(File aDirectory){

        if( aDirectory.list() == null ){
            String aPath = aDirectory.getPath();
            aPath = aPath.substring(0,aPath.length()-1);
            aDirectory = new File( aPath );
            this.setLabelString(aPath);
        }

        String fileNames[] = aDirectory.list();

        Vector v = new Vector();

        for(int n = 0; n < fileNames.length; n++ ){
            v.addElement(fileNames[n]);
        }
        v = StringSorter.sort(v);


        this.setLabelString(aDirectory.getPath());

        this.newList();

        Enumeration enum = v.elements();
        while(enum.hasMoreElements()){

            this.directoryDialogList().addItem((String) enum.nextElement());

        }
    }

    private void setLabelString(String newLabel){

        int newLabelWidth = this.fontMetrics.stringWidth(newLabel);

        if(this.fontMetrics.stringWidth(newLabel) > 290){
            this.choiceLabel().setAlignment(Label.RIGHT);
        }
        else{
            this.choiceLabel().setAlignment(Label.LEFT);
        }
        this.choiceLabel().setText(newLabel);
    }

    public File getChosenFile(){

        File aFile = new File( this.choiceLabel().getText() );

        if( ((aFile.list() != null) & (!aFile.isDirectory())) ){
            aFile = new File( aFile.getPath() + separator + ".");
        }

        return aFile;
    }
    private void setWaitingCursor(){
        this.setCursor(new Cursor(Cursor.WAIT_CURSOR));
    }
    private void resetCursor(){
        this.setCursor(Cursor.getDefaultCursor());
    }
}
